/*

VMEDIT, a general multipage GTK based text editor.

Copyright (C) 2005  Vladimir Martinek

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <string.h>
#ifdef GPEBUILD
#include <gpe/init.h>
#include <gpe/errorbox.h>
#include <gpe/pixmaps.h>
#include <gpe/picturebutton.h>
#include <gpe/question.h>
#include <gpe/spacing.h>
#endif
#include "vmedit.h"

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL,           0, "<Branch>" },
  { "/File/tear",  NULL,         NULL,           0, "<Tearoff>" },
  { "/File/_New",     "<control>N", create_new_file,    0, "<StockItem>", GTK_STOCK_NEW },
  { "/File/_Open",    "<control>O", open_file,    0, "<StockItem>", GTK_STOCK_OPEN },
  { "/File/_Save",    "<control>S", save_file_menu,    0, "<StockItem>", GTK_STOCK_SAVE },
  { "/File/Save _As", NULL,         save_as_file_menu,           0, "<StockItem>", GTK_STOCK_SAVE_AS},
  { "/File/Close", NULL,         close_file,           0, "<StockItem>", GTK_STOCK_CLOSE},
  { "/File/sep1",     NULL,         NULL,           0, "<Separator>" },
  { "/File/_Quit",    "<CTRL>Q", quit, 0, "<StockItem>", GTK_STOCK_QUIT },
  { "/_Edit",      NULL,         NULL,           0, "<Branch>" },
  { "/Edit/tear",  NULL,         NULL,           0, "<Tearoff>" },
  { "/Edit/Copy", NULL,         edit_copy,           0, "<StockItem>", GTK_STOCK_COPY },
  { "/Edit/Cut", NULL,         edit_cut,           0, "<StockItem>", GTK_STOCK_CUT },
  { "/Edit/Paste", NULL,         edit_paste,           0, "<StockItem>", GTK_STOCK_PASTE }
  //  { "/_Options",      NULL,         NULL,           0, "<Branch>" },
  //{ "/Options/tear",  NULL,         NULL,           0, "<Tearoff>" },
  //{ "/Options/Wrap", NULL,         set_wrap,           0, "<CheckItem>"},
  //{ "/Options/Show Toolbar", NULL, set_toolbar,           0, "<CheckItem>"},

  //  { "/Edit/sep1",     NULL,         NULL,           0, "<Separator>" },
  //{ "/Edit/Search", NULL,         NULL,           0, "<StockItem>", GTK_STOCK_FIND },
  //{ "/Edit/Replace", NULL,         NULL,           0, "<StockItem>", GTK_STOCK_FIND_AND_REPLACE },
  //{ "/_Help",         NULL,         NULL,           0, "<LastBranch>" },
  //{ "/Help/tear",  NULL,         NULL,           0, "<Tearoff>" },
  //{ "/_Help/About",   NULL,         NULL,           0, "<StockItem>", GTK_STOCK_HELP }
};

static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

/* Returns a menubar widget made from the above menu */
static GtkWidget *get_menubar_menu( GtkWidget  *window )
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  /* Make an accelerator group (shortcut keys) */
  accel_group = gtk_accel_group_new ();

  /* Make an ItemFactory (that makes a menubar) */
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
  //gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory,"/Options/Wrap")),TRUE);
  //gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory,"/Options/Show Toolbar")),TRUE);


  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* Finally, return the actual menu bar created by the item factory. */
  return gtk_item_factory_get_widget (item_factory, "<main>");
}

GtkWidget *notebook;
GtkWidget *window;
GtkWidget * toolbar;
GPtrArray  *managedFiles;
gint    managedFilesCount;
GtkWidget *fileSelection;



/* You have to start somewhere */
int main( int argc,
          char *argv[] )
{

  GtkWidget *main_vbox;
  GtkWidget *menubar;
  //GtkWidget *handlebox;
  //GtkWidget *statusBar;


  /* Initialize GTK */

#ifdef GPEBUILD
  if (gpe_application_init (&argc, &argv) == FALSE)
    return -1;
#else
  gtk_init (&argc, &argv);
#endif
  /* Make a window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (window), "delete-event",
                    G_CALLBACK (quit),
                    NULL);
  gtk_window_set_title (GTK_WINDOW(window), "Text Editor");
  gtk_window_set_default_size(GTK_WINDOW(window), 200, 200);
 
  /* Make a vbox to put the three menus in */
  main_vbox = gtk_vbox_new (FALSE, 1);
  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 1);
  gtk_container_add (GTK_CONTAINER (window), main_vbox);
 
  /* Get the three types of menu */
  /* Note: all three menus are separately created, so they are not the
     same menu */
  menubar = get_menubar_menu (window);
  //handlebox = gtk_handle_box_new ();
  toolbar = get_toolbar();
  //statusBar=gtk_statusbar_new();

  // Create notebook
  notebook = gtk_notebook_new();
  gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook),TRUE);



  /* Pack it all together */
  //gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
  //gtk_box_pack_start (GTK_BOX (main_vbox),handlebox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox),toolbar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox),notebook , TRUE, TRUE, 0);
  //gtk_box_pack_start (GTK_BOX (main_vbox),statusBar , FALSE, TRUE, 0);


  g_signal_connect (G_OBJECT (notebook), "switch-page",
                    G_CALLBACK (page_change),
                    NULL);

  // Create first page

  managedFiles = g_ptr_array_new();
  managedFilesCount=0;
  fileSelection = NULL;

  
  
  //create_new_file();
  /* Show the widgets */
  gtk_widget_show_all (window);
  /* Finished! */
  gtk_main ();
 
  return 0;
}

void create_new_file(void) {

  GtkWidget *view;
  GtkWidget *label;
  GtkWidget *scrolledWindow;
  GtkTextBuffer *buffer; 
  GtkAdjustment *hadjust, *vadjust;
  ManagedFile * fileDetails;
  int pagenum;


  view = gtk_text_view_new ();
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_set_text (buffer, "", -1);
  hadjust = GTK_ADJUSTMENT(gtk_adjustment_new(1.0,  -32000.0, 32000.0, 1.0, 5.0, 5.0));
  vadjust = GTK_ADJUSTMENT(gtk_adjustment_new(1.0,  -32000.0, 32000.0, 1.0, 5.0, 5.0));
  scrolledWindow=gtk_scrolled_window_new(hadjust,vadjust);
  gtk_container_add (GTK_CONTAINER (scrolledWindow), view);
  label = gtk_label_new ("New File");
  pagenum = gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scrolledWindow, label);
  gtk_widget_show_all(notebook);

  if (pagenum >= 0) {
   // Add a new item to the array
    fileDetails = g_new0(ManagedFile,1);
    fileDetails->textView=view;
    fileDetails->fileName=NULL;
    g_ptr_array_add(managedFiles,fileDetails);
    managedFilesCount++;
    gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook),pagenum);
  }

}

void edit_copy(void) {

  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"copy-clipboard");

}

void edit_paste(void) {
  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"paste-clipboard");

}

void edit_cut(void) {
  if (managedFilesCount <= 0) return; // all removed
  g_signal_emit_by_name(get_current_view(),"cut-clipboard");

}

void open_file(void) {

  //gchar *fileName;
  
  fileSelection = gtk_file_selection_new("Open File");

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fileSelection)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (do_open_file), NULL);
  
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fileSelection)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) fileSelection);
  gtk_window_set_transient_for(GTK_WINDOW(fileSelection), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);

  gtk_widget_show (fileSelection);
  
  /*  fileSelection = gtk_file_chooser_dialog_new(
	     "Open File",GTK_WINDOW(window),
	     GTK_FILE_CHOOSER_ACTION_OPEN,
	     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
	     NULL);

  if (gtk_dialog_run (GTK_DIALOG (fileSelection)) == GTK_RESPONSE_ACCEPT) {
    fileName = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fileSelection));
    // Open file, if fails, free the fileName. If succeeded,
    // then leave it, will be freed when closing file.
    if (!do_open_file (fileName)) g_free (fileName);
    }

    gtk_widget_destroy (fileSelection);*/


}

gboolean save_file(void) {

  ManagedFile * fileDetail;
  fileDetail = getFileDetail(get_current_page());
  
  if (fileDetail->fileName == NULL)
    return save_as_file();
  else
    return do_save_file(fileDetail->fileName);

}

gboolean save_as_file(void) {

  GtkWidget *fileSelection;
  gchar *fileName;
  GtkWidget * messageDialog;
  gboolean doSave;
  gboolean res;

  res = FALSE;
  /*fileSelection = gtk_file_chooser_dialog_new(
	     "Save File As",GTK_WINDOW(window),
	     GTK_FILE_CHOOSER_ACTION_SAVE,
	     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	     GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
	     NULL);*/

  fileSelection = gtk_file_selection_new ("Save as...");
  gtk_window_set_transient_for(GTK_WINDOW(fileSelection), GTK_WINDOW(window));
  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);

  
  if (gtk_dialog_run (GTK_DIALOG (fileSelection)) == GTK_RESPONSE_OK) {
    
    //fileName = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fileSelection));
    fileName = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fileSelection)));

    // Check whether the file exists, if so, then raise a question

    doSave=FALSE;
    if (g_file_test(fileName,G_FILE_TEST_EXISTS)) {

#ifdef GPEBUILD
      
      doSave = gpe_question_ask ("Current file is modified, save?","Question", 
				 "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL);
#else
      messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_YES_NO,"File %s already exists.\nOverwrite?",fileName);
      doSave=(gtk_dialog_run (GTK_DIALOG (messageDialog))==GTK_RESPONSE_YES);
      gtk_widget_destroy (messageDialog);
#endif
    } else
      doSave = TRUE;


    if (doSave) {
      // Save file, if fails, free the fileName. If succeeded,
      // then leave it, will be freed when closing file.
      res = do_save_file(fileName);
      if (!res)
	g_free(fileName);
    } else
      g_free(fileName);

  }
  gtk_widget_destroy (fileSelection);
  return res;

}


gboolean do_open_file () {

  gchar *text, *utf8_text, *fileName;
  gsize length;
  GError *error = NULL;
  gboolean ret = FALSE;
  text=NULL;
  utf8_text=NULL;
  GtkTextBuffer * buffer;

  fileName = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fileSelection)));
  gtk_widget_destroy(fileSelection);
  if (g_file_get_contents (fileName, &text, &length, NULL)) {
    utf8_text = g_convert (text, length, "UTF-8", "ISO-8859-1", 
			   NULL, NULL, &error);
    if (error != NULL) {
      g_print("Error: Couldn't convert file %s to UTF-8\n", fileName);
      g_error_free (error);
    }
    else {
      create_new_file();
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(get_current_view()));
      gtk_text_buffer_set_text(buffer,utf8_text,length);
      set_page_title(get_current_page(),basename(fileName));
      set_detail_file_name(get_current_page(),fileName);
      gtk_text_buffer_set_modified(buffer,FALSE);
      ret = TRUE;
    }
    
  }
  else 
    g_print("Error: Unable to read file %s\n", fileName);

  if (text!=NULL) g_free(text);

  if (!ret) g_free(fileName);
  
  return ret;
}


gboolean do_save_file (gchar * fileName) {

  gchar *text = NULL;
  gchar *utf8_text = NULL;
  GError *error = NULL;
  GtkTextIter start, end;
  GtkTextBuffer *buffer;
  gsize length;
  gboolean ret = FALSE;
  FILE * fp;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(get_current_view()));

  gtk_text_buffer_get_bounds(buffer, &start, &end);

  utf8_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);

  text = g_convert (utf8_text,-1,"ISO-8859-1","UTF-8",
		    NULL, NULL, &error);

  if (error != NULL) {
    g_print(error->message);
    g_error_free (error);
  }
  else {
    // Actual Write
    if ( (fp = fopen(fileName, "w")) == NULL)
      {
	g_print("Error: Couldn't open file %s for writing.\n", fileName);
      } else {
	length = strlen(text);
        fwrite (text, 1, length, fp);
        fclose (fp);
	set_page_title(get_current_page(),basename(fileName));
	set_detail_file_name(get_current_page(),fileName);
	gtk_text_buffer_set_modified(buffer,FALSE);
	ret = TRUE;
      }
  }
  return ret;
}

void close_file(void) {

  guint pageNumber;
  ManagedFile * fileDetail;
  GtkResponseType resp;

  if (managedFilesCount <= 0) return; // all removed
  
  pageNumber = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  fileDetail = getFileDetail(pageNumber);


  resp = ask_before_closing(pageNumber);
  if (resp == GTK_RESPONSE_CANCEL) return; // cowardly running away

  if (resp == GTK_RESPONSE_YES)
    if (!save_file()) return; // save file failed, return
      
  
  g_ptr_array_remove_index(managedFiles,pageNumber);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), pageNumber);
  if (fileDetail->fileName != NULL) g_free(fileDetail->fileName);
  managedFilesCount --;

}

ManagedFile * getFileDetail(guint pageNumber) {
  // find file detail for selected page number
  // Returns pointer to that detail or NULL if
  //not found

  ManagedFile * managedFile;

  if (pageNumber >= managedFilesCount) return NULL;
  
  managedFile = g_ptr_array_index (managedFiles,pageNumber);
  return managedFile;
}


void page_change (GtkNotebook *notebook,GtkNotebookPage *page,
		  guint page_num,gpointer user_data) {


  // g_print("Page Changed To: %d\n",page_num);

  //currentPage=page_num;

}

void set_page_title(guint pageNumber, gchar * pageTitle) {

  GtkWidget * labelItem;
  GtkWidget * currentPage;

  currentPage=gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),pageNumber);
  labelItem = gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),currentPage);
  gtk_label_set_text(GTK_LABEL(labelItem),pageTitle);

}

GtkWidget * get_current_view() {

  guint pageNumber;

  ManagedFile * fileDetail;
  pageNumber = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  
  fileDetail = getFileDetail(pageNumber);
  
  return fileDetail->textView;
  
}

guint get_current_page() {

  return gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

}

void set_detail_file_name(guint pageNumber, gchar * fileName) {


  ManagedFile * fileDetail;
  fileDetail = getFileDetail(pageNumber);

  if (fileDetail->fileName == fileName) return; // reassigned, nothing to do
  
  // release previously allocated file name
  if (fileDetail->fileName != NULL) g_free(fileDetail->fileName);
  fileDetail->fileName = fileName;
  
}

GtkResponseType ask_before_closing(guint pageNumber) {
  
  ManagedFile * fileDetail;
  GtkTextBuffer * buffer;
  GtkWidget * messageDialog;
  GtkResponseType rsp;

  fileDetail = getFileDetail(pageNumber);

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(fileDetail->textView));
  rsp = GTK_RESPONSE_NO;


  if (gtk_text_buffer_get_modified(buffer)) {
    gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook),pageNumber);
#ifdef GPEBUILD

    if (gpe_question_ask ("Current file is modified, save?","Question", 
			  "question", "!gtk-no", NULL, "!gtk-yes", NULL, NULL))
      rsp = GTK_RESPONSE_YES;
#else
    
    
    messageDialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
					   GTK_BUTTONS_NONE,
					   "File is modified.Do you wish to save it first?");
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),"Yes",GTK_RESPONSE_YES);
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),"No",GTK_RESPONSE_NO);
    gtk_dialog_add_button(GTK_DIALOG(messageDialog),"Cancel",GTK_RESPONSE_CANCEL);
    
    rsp = gtk_dialog_run (GTK_DIALOG (messageDialog));
    gtk_widget_destroy (messageDialog);

#endif

  }
  return rsp;

}

void save_as_file_menu(void) {
  
  if (managedFilesCount <= 0) return; // all removed
  save_as_file();
}

void save_file_menu(void) {
  if (managedFilesCount <= 0) return; // all removed
  save_file();
}

GtkWidget * get_toolbar() {

  GtkWidget * tb;
  GtkWidget *toolbar_icon;

  tb = gtk_toolbar_new ();
  gtk_toolbar_set_orientation (GTK_TOOLBAR (tb), GTK_ORIENTATION_HORIZONTAL);
  //gtk_toolbar_set_style (GTK_TOOLBAR (tb), GTK_TOOLBAR_BOTH);
  gtk_container_set_border_width (GTK_CONTAINER (tb), 0);


  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_toolbar_append_item (GTK_TOOLBAR (tb),"New", 
			   "New File","New File",
			   toolbar_icon, 
			   GTK_SIGNAL_FUNC(create_new_file), NULL);

  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_toolbar_append_item (GTK_TOOLBAR (tb),"Open", 
			   "Open File","Open File",
			   toolbar_icon, 
			   GTK_SIGNAL_FUNC(open_file), NULL);

  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_toolbar_append_item (GTK_TOOLBAR (tb),"Save", 
			   "Save File","Save File",
			   toolbar_icon, 
			   GTK_SIGNAL_FUNC(save_file_menu), NULL);

  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_toolbar_append_item (GTK_TOOLBAR (tb),"Quit", 
			   "Quit","Quit",
			   toolbar_icon, 
			   GTK_SIGNAL_FUNC(quit), NULL);


  return tb;

}
void quit(void) {
  
  int i;
  GtkResponseType resp;
  for (i=0;i<managedFilesCount;i++) {
    resp = ask_before_closing(i);
    if (resp == GTK_RESPONSE_CANCEL) return; // cowardly running away
    
    if (resp == GTK_RESPONSE_YES)
      if (!save_file()) return; // save file failed, return
  }
  gtk_exit(0);
}

/*void set_wrap (GtkCheckMenuItem *checkmenuitem, gpointer user_data) {
  g_print("In set_wrap\n");
}

void set_toolbar (GtkCheckMenuItem *checkmenuitem, gpointer user_data) {
  
  g_print("In set_toolbar\n");
  //gtk_check_menu_item_get_active(checkmenuitem);

  if (checkmenuitem->active)
    gtk_widget_show(toolbar);
  else
  gtk_widget_hide(toolbar);
}

*/
